// Lua interface
// Copyright (c) 2012 by Kasji

#ifndef __LUA_H__
#define __LUA_H__

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <map>
#include <list>
#ifdef __cplusplus
#include <lua5.3/lua.hpp>
#else
//extern "C" {
#include <lua5.3/lualib.h>
#include <lua5.3/lauxlib.h>
//}
#endif // __cplusplus
#include "mud.h"


// Easy macros
#define DECLARE_LUA_FUN(fun)		extern "C" int (fun)(lua_State * L)

#define	L_PUSH_STR(a, b)                \
        do { if ((b) != NULL) {         \
        lua_pushstring((a), (b));       \
        lua_setfield((a), -2, (#b)); }  \
        } while (0)

#define	L_PUSH_NUM(a, b)                \
        do {                            \
        lua_pushnumber((a), (b));       \
        lua_setfield((a), -2, (#b));     \
        } while (0)

#define L_PUSH_UD(a, b)				\
	do {					\
	lua_pushlightuserdata((a), (void*)(b));	\
	lua_setfield((a), -1, (#b));		\
	} while(0)

#define	L_GLOBAL_FUN(a, b, c)		\
	do {				\
	lua_pushcfunction((a), (b));	\
	lua_setglobal((a), (c));	\
	} while (0)

// Character macros
#define L_PUSH_CH_STR(a, b)             \
        do { if (ch->(b) != NULL) {     \
        lua_pushstring((a), ch->(b));   \
        lua_setfield((a), -2, (#b)); }  \
        } while (0)

#define L_PUSH_CH_NUM(a, b)             \
        do {                            \
        lua_pushnumber((a), (b));       \
        lua_setfield((a), -2, (#b));     \
        } while (0)

// Object macros
#define L_PUSH_OBJ_STR(a, b)            \
        do { if (obj->(b) != NULL) {    \
        lua_pushstring((a), obj->(b));  \
        lua_setfield((a), -2, (#b)); }  \
        } while (0)

#define L_PUSH_OBJ_NUM(a, b)            \
        do {                            \
        lua_pushnumber((a), obj->(b));  \
        lua_setfield((a), -2, (#b));     \
        } while (0)

// Room macros
#define L_PUSH_ROOM_STR(a, b)           \
        do { if (room->(b) != NULL) {   \
        lua_pushstring((a), room->(b)); \
        lua_setfield((a), -2, (#b)); }  \
        } while (0)

#define L_PUSH_ROOM_NUM(a, b)           \
        do {                            \
        lua_pushnumber((a), room->(b)); \
        lua_setfield((a), -2, (#b));    \
        } while (0)


// Return types.
#define	RET_OK	0
#define	RET_ERR	1

// Lua directories
#define LUA_DIR				"../lua/"
#define LUA_CMD_DIR			LUA_DIR "commands/"
#define LUA_WRLD_DIR			LUA_DIR "world/"
#define LUA_STARTUP_FILE	"startup.lua"

// Environment variables
#define MUDLIB_NAME	"mud"
#define CHAR_ID		"__character_id"
#define OWNER_ID		"__owner_id"

typedef struct lua_coroutine L_COROUTINE;
typedef struct lua_updater L_UPDATER;

class lua_coroutine : public tObject<L_COROUTINE>
{
public:
	lua_coroutine(const char *, uint64);
	~lua_coroutine();
	bool operator==(const lua_coroutine &rhs);

	int run(const char *, int);
	int resume();
	const char * owner_type();
	const char * func();
	uint64 owner_id();
	const timespec& created();
	bool started();
	int status();
	lua_State * L();

private:
	lua_State * m_L;
	struct timespec m_created;
	uint64 m_owner_id;
	char m_owner_type[10];
	char m_func[32];
	int m_state;
	bool m_started;
	int m_nargs;
};

typedef std::list<uint64> L_COR_LIST;
typedef L_COR_LIST::iterator L_COR_IT;

class lua_updater
{
public:
	lua_updater();
	~lua_updater();
	void update();
	L_COROUTINE * newthread(const char *, uint64);

private:
	void killthread(uint64);
	L_COR_IT killthread(L_COR_IT);

	L_COR_LIST m_coroutine_list;
};

int luaRegisterEnviron(lua_State * L);
void log_lua(const char * str, ...);
void lua_command(CHAR_DATA * ch, const char * command, const char * argument);
void lua_registerMud(lua_State * L);
void lua_pushCH(lua_State * L, CHAR_DATA * ch);
void lua_pushRoom(lua_State * L, ROOM_INDEX_DATA * room);
void lua_pushObj(lua_State * L, OBJ_DATA * obj);
char * lua_check_filename(char * filename);

void lua_mspeech_trigger(const char *txt, CHAR_DATA * actor);
void lua_ospeech_trigger(const char *txt, CHAR_DATA * actor);
void lua_rspeech_trigger(const char *txt, CHAR_DATA * actor);
void lua_mact_trigger(char *buf, CHAR_DATA * mob, CHAR_DATA * ch, OBJ_DATA * obj, void *vo);
void lua_ract_trigger(char *buf, ROOM_INDEX_DATA * room, CHAR_DATA * ch, OBJ_DATA * obj, void *vo);
void lua_oact_trigger(char *buf, OBJ_DATA * mobj, CHAR_DATA * ch, OBJ_DATA * obj, void *vo);
void lua_mentry_trigger(CHAR_DATA * mob);
void lua_renter_trigger(CHAR_DATA * ch);
void lua_mgreet_trigger(CHAR_DATA * ch);
void lua_ogreet_trigger(CHAR_DATA * ch);
void lua_rsleep_trigger(CHAR_DATA * ch);
void lua_rrest_trigger(CHAR_DATA * ch);
void lua_mgive_trigger(CHAR_DATA * mob, CHAR_DATA * ch, OBJ_DATA * obj);
void lua_mbribe_trigger(CHAR_DATA * mob, CHAR_DATA * ch, int amount);
void lua_rlook_trigger(CHAR_DATA * ch);
void lua_mlook_trigger(CHAR_DATA * ch, CHAR_DATA * mob);
void lua_oexamine_trigger(CHAR_DATA * ch, OBJ_DATA * obj);
bool lua_ouse_trigger(CHAR_DATA * ch, OBJ_DATA * obj, CHAR_DATA * vict);
void lua_owear_trigger(CHAR_DATA * ch, OBJ_DATA * obj);
void lua_oget_trigger(CHAR_DATA * ch, OBJ_DATA * obj);
void lua_odrop_trigger(CHAR_DATA * ch, OBJ_DATA * obj);
void lua_odamage_trigger(CHAR_DATA * ch, OBJ_DATA * obj);
void lua_oremove_trigger(CHAR_DATA * ch, OBJ_DATA * obj);
void lua_osac_trigger(CHAR_DATA * ch, OBJ_DATA * obj);
void lua_rfight_trigger(CHAR_DATA * ch);
void lua_mfight_trigger(CHAR_DATA * mob, CHAR_DATA * ch);
void lua_rdeath_trigger(CHAR_DATA * killer, CHAR_DATA * mob);
void lua_mdeath_trigger(CHAR_DATA * killer, CHAR_DATA * mob);
void lua_mhitprcnt_trigger(CHAR_DATA * ch, CHAR_DATA * mob);
int lua_mcustom_trigger( const char *command, const char *argument, CHAR_DATA * ch );
int lua_rcustom_trigger( const char *command, const char *argument, CHAR_DATA * ch );
int lua_ocustom_trigger( const char *command, const char *argument, CHAR_DATA * ch );
void lua_mrandom_trigger(CHAR_DATA * mob);
void lua_rrandom_trigger(CHAR_DATA * ch);
void lua_orandom_trigger(OBJ_DATA * obj);
void lua_mtime_trigger(CHAR_DATA * mob);
void lua_rtime_trigger(CHAR_DATA * mob);
void lua_mhour_trigger(CHAR_DATA * mob);
void lua_rhour_trigger(CHAR_DATA * mob);
void lua_mwordlist_check(const char *arg, CHAR_DATA * mob, CHAR_DATA * actor, OBJ_DATA * obj, void *vo, int type);

#endif // __LUA_H__

