// Lua implementation.
// Copyright (c) 2012 by Kasji.

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <setjmp.h>
#include "mud.h"
#include "lua.h"

extern lua_State * g_luaState;
extern jmp_buf g_luaEnv;
extern tPointerOwner<L_COROUTINE> coroutine_map;

DECLARE_LUA_FUN(luaPanic);

lua_coroutine::lua_coroutine(const char * type, uint64 id) : tObject<L_COROUTINE>(coroutine_map)
{
	clock_gettime(CLOCK_MONOTONIC, &(this->m_created));
	snprintf(this->m_owner_type, 10, "%s", type);
	this->m_owner_id = id;
	if (g_luaState)
	{
		this->m_L = lua_newthread(g_luaState);
		lua_atpanic(this->m_L, luaPanic);
		lua_gc(this->m_L, LUA_GCSTOP, 0);
	}
	else
		this->m_L = NULL;
	this->m_state = 0;
	this->m_started = FALSE;
	return;
}

lua_coroutine::~lua_coroutine()
{
	if (this->m_L)
	{
		lua_gc(this->m_L, LUA_GCRESTART, 0);
		lua_gc(this->m_L, LUA_GCCOLLECT, 0);
	}
}

int lua_coroutine::run(const char * funcname, int nargs)
{
	int jmp;

	jmp = setjmp(g_luaEnv);
	if (jmp == 1)
	{
		log_lua("Long jump successful.");
		return 0;
	}

	snprintf(this->m_func, 32, "%s", funcname);

   lua_getglobal(this->m_L, funcname);
   if (!lua_isfunction(this->m_L, -1))
   {
      log_lua("lua_coroutine::run: Function not found: %s", funcname);
//      lua_settop(g_luaState, 0);
      return 1;
   }
	if (nargs > 0)
		lua_insert(this->m_L, (-1 - nargs));

	this->m_state = lua_resume(this->m_L, NULL, nargs);
	this->m_started = TRUE;
	return this->m_state;
}

int lua_coroutine::resume()
{
   int jmp;

	if (this->m_started == FALSE || this->m_state != LUA_YIELD)
		return 0;

   jmp = setjmp(g_luaEnv);
   if (jmp == 1)
   {
      log_lua("Long jump successful.");
      return 0;
   }

	// If coroutine.yield returned any values, they need to be popped.
//	while (lua_gettop(this->m_L) > (1 + this->m_nargs))
//		lua_pop(this->m_L, -1);

	this->m_state = lua_resume(this->m_L, NULL, 0);
	return this->m_state;
}

int lua_coroutine::status()
{
	if (this->m_L)
		this->m_state = lua_status(this->m_L);
	return this->m_state;
}

bool lua_coroutine::started()
{
	return this->m_started;
}

const timespec& lua_coroutine::created()
{
	return this->m_created;
}

uint64 lua_coroutine::owner_id()
{
	return this->m_owner_id;
}

const char * lua_coroutine::owner_type()
{
	return this->m_owner_type;
}

lua_State * lua_coroutine::L()
{
	return this->m_L;
}

const char * lua_coroutine::func()
{
	return this->m_func;
}

bool lua_coroutine::operator==(const lua_coroutine &rhs)
{
	return this->GetId().Value() == rhs.GetId().Value();
}

