// Lua interface.
// Copyright (c) 2012 by Kasji.

#include <stdio.h>
#include <string.h>
#include <time.h>
#include "lua.h"

extern tPointerOwner<CHAR_DATA> char_map;
extern tPointerOwner<OBJ_DATA> obj_map;
extern tPointerOwner<ROOM_INDEX_DATA> room_map;

extern "C" int L_send_to_char(lua_State * L)
{
	CHAR_DATA * ch;
	const char * arg;
//	char buf[MAX_STRING_LENGTH];
	uint64 id;

	if (!L)
	{
		log_lua("L_send_to_char: FATAL error: L == NULL");
		return 0;
	}

	if (lua_gettop(L) != 2)
	{
		log_lua("L_send_to_char: Invalid number of arguments.");
		luaL_error(L, "L_send_to_char: Invalid number of arguments.");
		return 0;
	}

	id = (uint64) lua_touserdata(L, -2);
	ch = char_map[id];
	if (ch == NULL)
	{
		log_lua("L_send_to_char: NULL ch.");
		luaL_error(L, "L_send_to_char: NULL ch.");
		return 0;
	}

	if (IS_NPC(ch))
		return 0;

	arg = lua_tostring(L, -1);
	if (arg == NULL)
	{
		log_lua("L_send_to_char: NULL argument.");
		luaL_error(L, "L_send_to_char: NULL argument.");
		return 0;
	}

//	snprintf(buf, MAX_STRING_LENGTH, "%s\n\r", arg);

	send_to_char(arg, ch);

	return 0;
}

extern "C" int L_get_char_world(lua_State * L)
{
   const char * name = NULL;
   CHAR_DATA * ch = NULL, * och = NULL;
	uint64 id = 0;
	int i = lua_gettop(L);

	if (i < 1 || i > 2)
	{
		luaL_error(L, "L_get_char_world: Invalid number of args, should be 1 or 2.");
		return 0;
	}

	if (i == 2 && lua_istable(L, -2))
	{
		lua_getfield(L, -2, "id");
	   id = (uint64)lua_touserdata(L, -1);
	   ch = char_map[id];
	}

	if (lua_isstring(L, i) || lua_isnumber(L, i))
	{
		name = lua_tostring(L, i);
		och = get_char_world(ch, name);
	}
	else
	{
		luaL_error(L, "L_get_char_world: Argument must be string or number.");
		return 0;
	}

	if (!och)
		lua_pushnil(L);
	else
		lua_pushCH(L, och);
   return 1;
}

extern "C" int L_get_char_room(lua_State * L)
{
   const char * name = NULL;
   CHAR_DATA * och = NULL;
	ROOM_INDEX_DATA * room;
   uint64 id = 0;
   int i = lua_gettop(L);

   if (i != 2)
   {
      luaL_error(L, "L_get_char_room: Invalid number of args, should be 2.");
      return 0;
   }

   if (lua_istable(L, -2))
   {
      lua_getfield(L, -2, "id");
      id = (uint64)lua_touserdata(L, -1);
      room = room_map[id];
   }
	else
	{
		i = lua_tonumber(L, -2);
		room = get_room_index(i);
	}

	if (!room)
	{
		luaL_error(L, "L_get_char_room: Room not found.");
		return 0;
	}

   if (lua_isstring(L, 2) || lua_isnumber(L, 2))
   {
      name = lua_tostring(L, 2);
      och = get_char_room_2(room, name);
   }
   else
   {
      luaL_error(L, "L_get_char_room: Argument must be string or number.");
      return 0;
   }

   if (!och)
      lua_pushnil(L);
   else
      lua_pushCH(L, och);
   return 1;
}

extern "C" int L_get_room_index(lua_State * L)
{
	ROOM_INDEX_DATA * room;

	if (lua_gettop(L) != 1 || !lua_isnumber(L, -1))
	{
		luaL_error(L, "L_get_room_index: Invalid syntax: get_room_index(vnum)");
		return 0;
	}

	room = get_room_index(lua_tonumber(L, -1));

	if (!room)
		lua_pushnil(L);
	else
		lua_pushRoom(L, room);
	return 1;
}

extern "C" int L_interpret(lua_State * L)
{
	CHAR_DATA * ch;
	uint64 id;
	const char * argument = lua_tostring(L, -1);

	if (lua_gettop(L) != 2 || !lua_istable(L, -2) || !lua_isstring(L, -1))
	{
		luaL_error(L, "L_interpret: Invalud syntax: interpret(ch, \"args\")");
		return 0;
	}

	lua_getfield(L, -2, "id");
	id = (uint64)lua_touserdata(L, -1);
	ch = char_map[id];

	if (!ch)
	{
		luaL_error(L, "L_interpret: NULL ch!");
		return 0;
	}

	interpret(ch, argument);

	return 0;
}

extern "C" int L_open_passage(lua_State * L)
{
   const char * arg1;
   ROOM_INDEX_DATA *targetRoom, *fromRoom;
   int targetRoomVnum, fromRoomVnum, exit_num;
   EXIT_DATA *pexit;

	if (lua_gettop(L) != 3 || !lua_isnumber(L, -3) || !lua_isnumber(L, -2) || !lua_isstring(L, -1))
	{
		luaL_error(L, "L_open_passage: Invalid syntax: open_passage(from vnum, to vnum, \"direction\")");
		return 0;
	}

	fromRoomVnum = lua_tonumber(L, -3);
	if ((fromRoom = get_room_index(fromRoomVnum)) == NULL)
	{
		luaL_error(L, "L_open_passage: 'from room' not found.");
		return 0;
	}

	targetRoomVnum = lua_tonumber(L, -2);
	if ((targetRoom = get_room_index(targetRoomVnum)) == NULL)
	{
		luaL_error(L, "L_open_passage: 'to room' not found.");
		return 0;
	}

	arg1 = lua_tostring(L, -1);
	for (exit_num = 0; exit_num <= MAX_DIR; exit_num++)
	{
		if (!str_cmp(arg1, dir_name[exit_num]))
			break;
	}

	if (exit_num > MAX_DIR)
	{
		luaL_error(L, "L_open_passage: 'direction' not found.");
		return 0;
	}

   if( ( pexit = get_exit( fromRoom, exit_num ) ) != NULL )
   {
		luaL_error(L, "L_open_passage: exit already exists.");
		return 0;
   }

   pexit = make_exit( fromRoom, targetRoom, exit_num );
   pexit->keyword = STRALLOC( "" );
   pexit->description = STRALLOC( "" );
   pexit->key = -1;
   pexit->exit_info = EX_PASSAGE;

   return 0;
}

extern "C" int L_close_passage(lua_State * L)
{
   const char * arg1;
   ROOM_INDEX_DATA *fromRoom;
   int fromRoomVnum, exit_num;
   EXIT_DATA *pexit;

   if (lua_gettop(L) != 2 || !lua_isnumber(L, -3) || !lua_isstring(L, -1))
   {
      luaL_error(L, "L_close_passage: Invalid syntax: open_passage(from vnum, to vnum, \"direction\")");
      return 0;
   }

   fromRoomVnum = lua_tonumber(L, -2);
   if( ( fromRoom = get_room_index( fromRoomVnum ) ) == NULL )
   {
		luaL_error(L, "L_close_passage: 'from room' not found.");
		return 0;
   }

	arg1 = lua_tostring(L, -1);
	for (exit_num = 0; exit_num <= MAX_DIR; exit_num++)
	{
		if (!str_cmp(arg1, dir_name[exit_num]))
			break;
	}

   if (exit_num > MAX_DIR)
   {
		luaL_error(L, "L_close_passage: 'direction' not found.");
		return 0;
   }

   if( ( pexit = get_exit( fromRoom, exit_num ) ) == NULL )
   {
      return 0;  /* already closed, ignore...  for rand_progs */
   }

   if(!IS_SET(pexit->exit_info, EX_PASSAGE))
   {
		luaL_error(L, "L_close_passage: Exit is not a passage.");
		return 0;
   }

   extract_exit( fromRoom, pexit );

   return 0;
}

extern "C" int L_say(lua_State * L)
{
	const char * argument = lua_tostring(L, -1);
	uint64 id;
	CHAR_DATA * ch;

   lua_getfield(L, -2, "id"); // ID of the character
   id = (uint64)lua_touserdata(L, -1);
   ch = char_map[id];
   // No character was found?
   if (!ch)
   {
		luaL_error(L, "L_say: NULL ch!");
		return 0;
   }

	do_say(ch, argument);
	return 0;
}

extern "C" int L_recho(lua_State * L)
{
	const char * argument = lua_tostring(L, -1);
	const char * type;
	ROOM_INDEX_DATA * room = NULL;
	CHAR_DATA * ch = NULL;
	uint64 id;

	lua_getfield(L, -2, "id");
	id = (uint64)lua_touserdata (L, -1);
	lua_getfield(L, -3, "type");
	type = luaL_optstring(L, -1, "unk");
	if (str_cmp(type, "room"))
	{
      luaL_error(L, "L_recho: Invalid argument. Must be a room.");
      return 0;
	}

	room = room_map[id];

	if (!room)
	{
		luaL_error(L, "L_recho: NULL room!");
		return 0;
	}

	for (ch = room->first_person; ch; ch = ch->next_in_room)
		send_to_char(argument, ch);

	return 0;
}

extern "C" int L_rechoaround(lua_State * L)
{
   const char * argument = lua_tostring(L, -1);
   const char * type;
   ROOM_INDEX_DATA * room = NULL;
   CHAR_DATA * ch = NULL;
	CHAR_DATA * vict = NULL;
   uint64 id;

   lua_getfield(L, -2, "id");
   id = (uint64)lua_touserdata (L, -1);
   lua_getfield(L, -3, "type");
   type = luaL_optstring(L, -1, "unk");
   if (str_cmp(type, "char"))
   {
      luaL_error(L, "L_rechoaround: Invalid argument. Must be a character.");
      return 0;
   }

   vict = char_map[id];
   if (!vict)
   {
      luaL_error(L, "L_rechoaround: NULL ch!");
      return 0;
   }

	room = vict->in_room;
   if (!room)
   {
      luaL_error(L, "L_rechoaround: NULL room!");
      return 0;
   }

   for (ch = room->first_person; ch; ch = ch->next_in_room)
		if (ch != vict)
	      send_to_char(argument, ch);

	return 0;
}

extern "C" int L_mudtime(lua_State * L)
{
	struct timeval t;
	double d;

	gettimeofday(&t, NULL);
	d = (double)t.tv_sec + ((double)t.tv_usec / 1000000);
	lua_pushnumber(L, d);
	return 1;
}

