// Lua interface.

#include "lua.h"

extern lua_State * g_luaState;

void mob_act_add( CHAR_DATA * mob );
void room_act_add( ROOM_INDEX_DATA * room );
void obj_act_add( OBJ_DATA * obj );

const char * const lua_hooks[] =
{
	"on_act", "on_speech", "on_random", "on_fighting", "on_hitpercent",
	"on_enter", "on_greet", "on_allgreet", "on_give", "on_bribe",
	"on_hour", "on_time", "on_wear", "on_remove", "on_sacrifice",
	"on_look", "on_examine", "on_custom", "on_get", "on_drop",
	"on_damage", "on_repair", "on_randiw", "on_speechiw", "on_pull",
	"on_push", "on_sleep", "on_rest", "on_leave", "on_script",
	"on_use"
};

#define MAX_PROG_TYPES	32

void lua_hook(const char * filename, int type, CHAR_DATA * ch, CHAR_DATA * mob, OBJ_DATA * obj, ROOM_INDEX_DATA * room)
{
   char path[MAX_INPUT_LENGTH];
   char func[MAX_INPUT_LENGTH];
	char area_filename[MAX_INPUT_LENGTH];
   int err, flag, i;

   if (!g_luaState)
   {
      log_lua("lua_command: FATAL error: g_luaState == NULL!");
      return;
   }

	if (!filename)
		return;

   // Push ch's id into the environment.
   lua_pushcfunction(g_luaState, luaRegisterEnviron);
	if (mob)
	{
	   lua_pushlightuserdata(g_luaState, (void*)mob->GetId().Value());
		snprintf(area_filename, MAX_INPUT_LENGTH, "%s", mob->pIndexData->area->filename);
	}
	else if (obj)
	{
	   lua_pushlightuserdata(g_luaState, (void*)obj->GetId().Value());
		snprintf(area_filename, MAX_INPUT_LENGTH, "%s", obj->pIndexData->area->filename);
	}
	else if (room)
	{
	   lua_pushlightuserdata(g_luaState, (void*)room->GetId().Value());
		snprintf(area_filename, MAX_INPUT_LENGTH, "%s", room->area->filename);
	}
   lua_call(g_luaState, 1, 0);
	lua_setglobal(g_luaState, OWNER_ID);
	lua_settop(g_luaState, 0);

   snprintf(path, MAX_INPUT_LENGTH, "%s%s/%s.lua", LUA_WRLD_DIR, area_filename, filename);
   if ((err = luaL_dofile(g_luaState, path)) != 0)
   {
      log_lua("lua_hook: luaL_dofile() error %d: %s\n\r   Path: %s", err, lua_tostring(g_luaState, -1), path);
      return;
   }

	flag = 1;
	for (i = 0; i < MAX_PROG_TYPES; i++)
	{
		if (type & flag)
			break;
		flag = flag << 1;
	}
	if (i >= MAX_PROG_TYPES)
	{
		log_lua("lua_hook: Invalid prog type.\n\r   Path: %s", path);
		return;
	}

   snprintf(func, MAX_INPUT_LENGTH, "%s", lua_hooks[i]);
   lua_getglobal(g_luaState, func);
   if (!lua_isfunction(g_luaState, -1))
   {
      log_lua("lua_hook: Function not found: %s", func);
      lua_settop(g_luaState, 0);
      return;
   }

   // Setup the ch metatable.
	if (ch)
	   lua_pushCH(g_luaState, ch);
	else
		lua_pushnil(g_luaState);
	lua_setglobal(g_luaState, "ch");

	if (mob)
		lua_pushCH(g_luaState, mob);
	else
		lua_pushnil(g_luaState);
	lua_setglobal(g_luaState, "mob");

	if (obj)
		lua_pushObj(g_luaState, obj);
	else
		lua_pushnil(g_luaState);
	lua_setglobal(g_luaState, "obj");

	if (room)
		lua_pushRoom(g_luaState, room);
	else
		lua_pushnil(g_luaState);
	lua_setglobal(g_luaState, "room");

   if ((err = lua_pcall(g_luaState, 0, 0, 0)) != 0)
   {
      log_lua("lua_command: lua_pcall() error: %d: %s\n\r   Path: %s", err, lua_tostring(g_luaState, -1), path);
      lua_settop(g_luaState, 0);
      return;
   }
   lua_settop(g_luaState, 0);

   return;
}

bool lua_keyword_check( const char *argu, const char *argl )
{
   char word[MAX_INPUT_LENGTH];
   char arg1[MAX_INPUT_LENGTH];
   char arg2[MAX_INPUT_LENGTH];
   unsigned int i;
   char *arg, *arglist;
   char *start, *end;

   strcpy( arg1, strlower( argu ) );
   arg = arg1;
   strcpy( arg2, strlower( argl ) );
   arglist = arg2;

   for( i = 0; i < strlen( arglist ); i++ )
      arglist[i] = LOWER( arglist[i] );
   for( i = 0; i < strlen( arg ); i++ )
      arg[i] = LOWER( arg[i] );
   if( ( arglist[0] == 'p' ) && ( arglist[1] == ' ' ) )
   {
      arglist += 2;
      while( ( start = strstr( arg, arglist ) ) )
         if( ( start == arg || *( start - 1 ) == ' ' )
             && ( *( end = start + strlen( arglist ) ) == ' ' || *end == '\n' || *end == '\r' || *end == '\0' ) )
            return TRUE;
         else
            arg = start + 1;
   }
   else
   {
      arglist = one_argument( arglist, word );
      for( ; word[0] != '\0'; arglist = one_argument( arglist, word ) )
         while( ( start = strstr( arg, word ) ) )
            if( ( start == arg || *( start - 1 ) == ' ' )
                && ( *( end = start + strlen( word ) ) == ' ' || *end == '\n' || *end == '\r' || *end == '\0' ) )
               return TRUE;
            else
               arg = start + 1;
   }
/*    bug( "don't match" ); */
   return FALSE;
}

void lua_mwordlist_check( const char *arg, CHAR_DATA * mob, CHAR_DATA * actor, OBJ_DATA * obj, void *vo, int type )
{
   char temp1[MAX_STRING_LENGTH];
   char temp2[MAX_INPUT_LENGTH];
   char word[MAX_INPUT_LENGTH];
   MPROG_DATA *mprg;
   char *list;
   char *start;
   char *dupl;
   char *end;
   unsigned int i;

   for( mprg = mob->pIndexData->mudprogs; mprg; mprg = mprg->next )
      if( mprg->type & type )
      {
         strcpy( temp1, mprg->arglist );
         list = temp1;
         for( i = 0; i < strlen( list ); i++ )
            list[i] = LOWER( list[i] );
         strcpy( temp2, arg );
         dupl = temp2;
         for( i = 0; i < strlen( dupl ); i++ )
            dupl[i] = LOWER( dupl[i] );
         if( ( list[0] == 'p' ) && ( list[1] == ' ' ) )
         {
            list += 2;
            while( ( start = strstr( dupl, list ) ) )
               if( ( start == dupl || *( start - 1 ) == ' ' )
                   && ( *( end = start + strlen( list ) ) == ' ' || *end == '\n' || *end == '\r' || *end == '\0' ) )
               {
//                  mprog_driver( mprg->comlist, mob, actor, obj, vo, FALSE );
						lua_hook(mprg->comlist, type, actor, mob, obj, NULL);
                  break;
               }
               else
                  dupl = start + 1;
         }
         else
         {
            list = one_argument( list, word );
            for( ; word[0] != '\0'; list = one_argument( list, word ) )
               while( ( start = strstr( dupl, word ) ) )
                  if( ( start == dupl || *( start - 1 ) == ' ' )
                      && ( *( end = start + strlen( word ) ) == ' ' || *end == '\n' || *end == '\r' || *end == '\0' ) )
                  {
//                     mprog_driver( mprg->comlist, mob, actor, obj, vo, FALSE );
							lua_hook(mprg->comlist, type, actor, mob, obj, NULL);
                     break;
                  }
                  else
                     dupl = start + 1;
         }
      }

   return;
}

void lua_owordlist_check( const char *arg, CHAR_DATA * mob, CHAR_DATA * actor,
                           OBJ_DATA * obj, void *vo, int type, OBJ_DATA * iobj )
{
   char temp1[MAX_STRING_LENGTH];
   char temp2[MAX_INPUT_LENGTH];
   char word[MAX_INPUT_LENGTH];
   MPROG_DATA *mprg;
   char *list;
   char *start;
   char *dupl;
   char *end;
   unsigned int i;

   for( mprg = iobj->pIndexData->mudprogs; mprg; mprg = mprg->next )
      if( mprg->type & type )
      {
         strcpy( temp1, mprg->arglist );
         list = temp1;
         for( i = 0; i < strlen( list ); i++ )
            list[i] = LOWER( list[i] );
         strcpy( temp2, arg );
         dupl = temp2;
         for( i = 0; i < strlen( dupl ); i++ )
            dupl[i] = LOWER( dupl[i] );
         if( ( list[0] == 'p' ) && ( list[1] == ' ' ) )
         {
            list += 2;
            while( ( start = strstr( dupl, list ) ) )
               if( ( start == dupl || *( start - 1 ) == ' ' )
                   && ( *( end = start + strlen( list ) ) == ' ' || *end == '\n' || *end == '\r' || *end == '\0' ) )
               {
//                  set_supermob( iobj );
//                  mprog_driver( mprg->comlist, mob, actor, obj, vo, FALSE );
//                  release_supermob(  );
						lua_hook(mprg->comlist, type, actor, mob, obj, NULL);
                  break;
               }
               else
                  dupl = start + 1;
         }
         else
         {
            list = one_argument( list, word );
            for( ; word[0] != '\0'; list = one_argument( list, word ) )
               while( ( start = strstr( dupl, word ) ) )
                  if( ( start == dupl || *( start - 1 ) == ' ' )
                      && ( *( end = start + strlen( word ) ) == ' ' || *end == '\n' || *end == '\r' || *end == '\0' ) )
                  {
//                     set_supermob( iobj );
//                     mprog_driver( mprg->comlist, mob, actor, obj, vo, FALSE );
//                     release_supermob(  );
							lua_hook(mprg->comlist, type, actor, mob, obj, NULL);
                     break;
                  }
                  else
                     dupl = start + 1;
         }
      }

   return;
}

void lua_rwordlist_check( const char *arg, CHAR_DATA * mob, CHAR_DATA * actor,
                           OBJ_DATA * obj, void *vo, int type, ROOM_INDEX_DATA * room )
{
   char temp1[MAX_STRING_LENGTH];
   char temp2[MAX_INPUT_LENGTH];
   char word[MAX_INPUT_LENGTH];
   MPROG_DATA *mprg;
   char *list;
   char *start;
   char *dupl;
   char *end;
   unsigned int i;

   if( actor && !char_died( actor ) && actor->in_room )
      room = actor->in_room;

   for( mprg = room->mudprogs; mprg; mprg = mprg->next )
      if( mprg->type & type )
      {
         strcpy( temp1, mprg->arglist );
         list = temp1;
         for( i = 0; i < strlen( list ); i++ )
            list[i] = LOWER( list[i] );
         strcpy( temp2, arg );
         dupl = temp2;
         for( i = 0; i < strlen( dupl ); i++ )
            dupl[i] = LOWER( dupl[i] );
         if( ( list[0] == 'p' ) && ( list[1] == ' ' ) )
         {
            list += 2;
            while( ( start = strstr( dupl, list ) ) )
               if( ( start == dupl || *( start - 1 ) == ' ' )
                   && ( *( end = start + strlen( list ) ) == ' ' || *end == '\n' || *end == '\r' || *end == '\0' ) )
               {
//                  rset_supermob( room );
//                  mprog_driver( mprg->comlist, mob, actor, obj, vo, FALSE );
//                  release_supermob(  );
						lua_hook(mprg->comlist, type, actor, mob, obj, room);
                  break;
               }
               else
                  dupl = start + 1;
         }
         else
         {
            list = one_argument( list, word );
            for( ; word[0] != '\0'; list = one_argument( list, word ) )
               while( ( start = strstr( dupl, word ) ) )
                  if( ( start == dupl || *( start - 1 ) == ' ' )
                      && ( *( end = start + strlen( word ) ) == ' ' || *end == '\n' || *end == '\r' || *end == '\0' ) )
                  {
//                     rset_supermob( room );
//                     mprog_driver( mprg->comlist, mob, actor, obj, vo, FALSE );
//                     release_supermob(  );
							lua_hook(mprg->comlist, type, actor, mob, obj, room);
                     break;
                  }
                  else
                     dupl = start + 1;
         }
      }

   return;
}

void lua_mpercent_check( CHAR_DATA * mob, CHAR_DATA * actor, int type )
{
   MPROG_DATA *mprg;

	if (!mob)
		return;

   for( mprg = mob->pIndexData->mudprogs; mprg; mprg = mprg->next )
      if( ( mprg->type & type ) && ( number_percent(  ) <= atoi( mprg->arglist ) ) )
      {
//         mprog_driver( mprg->comlist, mob, actor, obj, vo, FALSE );
			lua_hook(mprg->comlist, type, actor, mob, NULL, NULL);
//         if( type != GREET_PROG && type != ALL_GREET_PROG )
//            break;
      }

   return;
}

void lua_mtime_check( CHAR_DATA * mob, int type )
{
   MPROG_DATA *mprg;
   bool trigger_time;

   for( mprg = mob->pIndexData->mudprogs; mprg; mprg = mprg->next )
   {
      trigger_time = ( time_info.hour == atoi( mprg->arglist ) );

      if( !trigger_time )
      {
         if( mprg->triggered )
            mprg->triggered = FALSE;
         continue;
      }

      if( ( mprg->type & type ) && ( ( !mprg->triggered ) || ( mprg->type && HOUR_PROG ) ) )
      {
         mprg->triggered = TRUE;
//         mprog_driver( mprg->comlist, mob, actor, obj, vo, FALSE );
			lua_hook(mprg->comlist, type, NULL, mob, NULL, NULL);
      }
   }
   return;
}

bool lua_ouse_check( OBJ_DATA * obj, CHAR_DATA * actor, CHAR_DATA * mob, int type )
{
   MPROG_DATA *mprg;
	bool executed = FALSE;

   if (!obj)
      return executed;

   for( mprg = obj->pIndexData->mudprogs; mprg; mprg = mprg->next )
      if( ( mprg->type & type ) && ( number_percent(  ) <= atoi( mprg->arglist ) ) )
      {
         lua_hook(mprg->comlist, type, actor, mob, obj, NULL);
			executed = TRUE;
      }

   return executed;
}

void lua_opercent_check( OBJ_DATA * obj, CHAR_DATA * actor, int type )
{
   MPROG_DATA *mprg;

	if (!obj)
		return;

   for( mprg = obj->pIndexData->mudprogs; mprg; mprg = mprg->next )
      if( ( mprg->type & type ) && ( number_percent(  ) <= atoi( mprg->arglist ) ) )
      {
//         mprog_driver( mprg->comlist, mob, actor, obj, vo, FALSE );
         lua_hook(mprg->comlist, type, actor, NULL, obj, NULL);
//         if( type != GREET_PROG && type != ALL_GREET_PROG )
//            break;
      }

   return;
}

void lua_otime_check( OBJ_DATA * obj, int type )
{
   MPROG_DATA *mprg;
   bool trigger_time;

   for( mprg = obj->pIndexData->mudprogs; mprg; mprg = mprg->next )
   {
      trigger_time = ( time_info.hour == atoi( mprg->arglist ) );

      if( !trigger_time )
      {
         if( mprg->triggered )
            mprg->triggered = FALSE;
         continue;
      }

      if( ( mprg->type & type ) && ( ( !mprg->triggered ) || ( mprg->type && HOUR_PROG ) ) )
      {
         mprg->triggered = TRUE;
//         mprog_driver( mprg->comlist, mob, actor, obj, vo, FALSE );
         lua_hook(mprg->comlist, type, NULL, NULL, obj, NULL);
      }
   }
   return;
}

void lua_rpercent_check( ROOM_INDEX_DATA * room, CHAR_DATA * actor, int type )
{
   MPROG_DATA *mprg;

   if (!room)
      return;

   for( mprg = room->mudprogs; mprg; mprg = mprg->next )
      if( ( mprg->type & type ) && ( number_percent(  ) <= atoi( mprg->arglist ) ) )
      {
//         mprog_driver( mprg->comlist, mob, actor, obj, vo, FALSE );
         lua_hook(mprg->filename, type, actor, NULL, NULL, room);
//         if( type != GREET_PROG && type != ALL_GREET_PROG )
//            break;
      }

   return;
}

void lua_rtime_check( ROOM_INDEX_DATA * room, CHAR_DATA * actor, int type )
{
   MPROG_DATA *mprg;
   bool trigger_time;

   for( mprg = room->mudprogs; mprg; mprg = mprg->next )
   {
      trigger_time = ( time_info.hour == atoi( mprg->arglist ) );

      if( !trigger_time )
      {
         if( mprg->triggered )
            mprg->triggered = FALSE;
         continue;
      }

      if( ( mprg->type & type ) && ( ( !mprg->triggered ) || ( mprg->type && HOUR_PROG ) ) )
      {
         mprg->triggered = TRUE;
//         mprog_driver( mprg->comlist, mob, actor, obj, vo, FALSE );
         lua_hook(mprg->filename, type, actor, NULL, NULL, room);
      }
   }
   return;
}

void lua_mentry_trigger(CHAR_DATA * mob)
{
	if (IS_NPC(mob) && mob->pIndexData->progtypes & ENTRY_PROG)
		lua_mpercent_check(mob, NULL, ENTRY_PROG);

	return;
}

void lua_renter_trigger(CHAR_DATA * ch)
{
	ROOM_INDEX_DATA * vroom = ch->in_room;

	if (vroom->progtypes & ENTER_PROG)
		lua_rpercent_check(vroom, ch, ENTER_PROG);

	return;
}

void lua_mgreet_trigger(CHAR_DATA * ch)
{
   CHAR_DATA *vmob, *vmob_next;

   for( vmob = ch->in_room->first_person; vmob; vmob = vmob_next )
   {
      vmob_next = vmob->next_in_room;
      if( !IS_NPC( vmob ) || vmob->fighting || !IS_AWAKE( vmob ) )
         continue;

      /*
       * Don't let a mob trigger itself, nor one instance of a mob
       * trigger another instance.
       */
      if( IS_NPC( ch ) && ch->pIndexData == vmob->pIndexData )
         continue;

      if (vmob->pIndexData->progtypes & GREET_PROG)
         lua_mpercent_check( vmob, ch, GREET_PROG);
      if (vmob->pIndexData->progtypes & ALL_GREET_PROG)
         lua_mpercent_check( vmob, ch, ALL_GREET_PROG);
   }

   return;
}

void lua_ogreet_trigger(CHAR_DATA * ch)
{
	OBJ_DATA *vobj, *vobj_next;

	for (vobj = ch->in_room->first_content; vobj; vobj = vobj_next)
	{
		vobj_next = vobj->next_content;

		if (vobj->pIndexData->progtypes & GREET_PROG)
			lua_opercent_check(vobj, ch, GREET_PROG);
	}

	return;
}

void lua_mact_trigger( char *buf, CHAR_DATA * mob, CHAR_DATA * ch, OBJ_DATA * obj, void *vo )
{
   MPROG_ACT_LIST *tmp_act, *tmp_mal;
   MPROG_DATA *mprg;
   bool found = FALSE;

   if( IS_NPC( mob ) && IS_SET( mob->pIndexData->progtypes, ACT_PROG ) )
   {
      /*
       * Don't let a mob trigger itself, nor one instance of a mob
       * trigger another instance.
       */
      if( IS_NPC( ch ) && ch->pIndexData == mob->pIndexData )
         return;

      /*
       * make sure this is a matching trigger
       */
      for( mprg = mob->pIndexData->mudprogs; mprg; mprg = mprg->next )
         if( mprg->type & ACT_PROG && lua_keyword_check( buf, mprg->arglist ) )
         {
            found = TRUE;
            break;
         }
      if( !found )
         return;

      CREATE( tmp_act, MPROG_ACT_LIST, 1 );
/*
 * Losing the head of the list -Druid
 */
      if( mob->mpactnum > 0 )
      {
         tmp_mal = mob->mpact;

         while( tmp_mal->next != NULL )
            tmp_mal = tmp_mal->next;

         /*
          * Put at the end
          */
         tmp_mal->next = tmp_act;
      }
      else
         mob->mpact = tmp_act;

      tmp_act->next = NULL;
      tmp_act->buf = str_dup( buf );
      tmp_act->ch = ch;
      tmp_act->obj = obj;
      tmp_act->vo = vo;
      mob->mpactnum++;
      mob_act_add( mob );
   }
   return;
}

void lua_ract_trigger( char *buf, ROOM_INDEX_DATA * room, CHAR_DATA * ch, OBJ_DATA * obj, void *vo )
{
   if( room->progtypes & ACT_PROG )
   {
      MPROG_ACT_LIST *tmp_act, *tmp_mal;

      CREATE( tmp_act, MPROG_ACT_LIST, 1 );
/*
 * Losing the head of the list -Druid
 */
      if( room->mpactnum > 0 )
      {
         tmp_mal = room->mpact;

         while( tmp_mal->next != NULL )
            tmp_mal = tmp_mal->next;

         /*
          * Put at the end
          */
         tmp_mal->next = tmp_act;
      }
      else
         room->mpact = tmp_act;

      tmp_act->next = NULL;
      tmp_act->buf = str_dup( buf );
      tmp_act->ch = ch;
      tmp_act->obj = obj;
      tmp_act->vo = vo;
      room->mpactnum++;
      room_act_add( room );
   }
   return;
}

void lua_oact_trigger( char *buf, OBJ_DATA * mobj, CHAR_DATA * ch, OBJ_DATA * obj, void *vo )
{
   if( mobj->pIndexData->progtypes & ACT_PROG )
   {
      MPROG_ACT_LIST *tmp_act, *tmp_mal;

      CREATE( tmp_act, MPROG_ACT_LIST, 1 );
/*
 * Losing the head of the list -Druid
 */
      if( mobj->mpactnum > 0 )
      {
         tmp_mal = mobj->mpact;

         while( tmp_mal->next != NULL )
            tmp_mal = tmp_mal->next;

         /*
          * Put at the end
          */
         tmp_mal->next = tmp_act;
      }
      else
         mobj->mpact = tmp_act;

      tmp_act->next = NULL;
      tmp_act->buf = str_dup( buf );
      tmp_act->ch = ch;
      tmp_act->obj = obj;
      tmp_act->vo = vo;
      mobj->mpactnum++;
      obj_act_add( mobj );
   }
   return;
}

void lua_mbribe_trigger( CHAR_DATA * mob, CHAR_DATA * ch, int amount )
{

   char buf[MAX_STRING_LENGTH];
   MPROG_DATA *mprg;
   OBJ_DATA *obj;

   if( IS_NPC( mob ) && ( mob->pIndexData->progtypes & BRIBE_PROG ) )
   {
      /*
       * Don't let a mob trigger itself, nor one instance of a mob
       * trigger another instance.
       */
      if( IS_NPC( ch ) && ch->pIndexData == mob->pIndexData )
         return;

      obj = create_object( get_obj_index( OBJ_VNUM_MONEY_SOME ), 0 );
      sprintf( buf, obj->short_descr, amount );
      STRFREE( obj->short_descr );
      obj->short_descr = STRALLOC( buf );
      obj->value[0] = amount;
      obj = obj_to_char( obj, mob );
      mob->gold -= amount;

      for( mprg = mob->pIndexData->mudprogs; mprg; mprg = mprg->next )
         if( ( mprg->type & BRIBE_PROG ) && ( amount >= atoi( mprg->arglist ) ) )
         {
//            mprog_driver( mprg->comlist, mob, ch, obj, NULL, FALSE );
				lua_hook(mprg->comlist, BRIBE_PROG, ch, mob, NULL, NULL);
            break;
         }
   }

   return;

}

void lua_mdeath_trigger( CHAR_DATA * killer, CHAR_DATA * mob )
{
   if( IS_NPC( mob ) && ( mob->pIndexData->progtypes & DEATH_PROG ) )
   {
//      mprog_percent_check( mob, killer, NULL, NULL, DEATH_PROG );
		lua_mpercent_check(mob, killer, DEATH_PROG);
   }
   death_cry( mob );
   return;
}

void lua_rdeath_trigger( CHAR_DATA * killer, CHAR_DATA * mob )
{
   if( mob->in_room->progtypes & RDEATH_PROG )
   {
//      rset_supermob( ch->in_room );
//      rprog_percent_check( supermob, ch, NULL, NULL, RDEATH_PROG );
//      release_supermob(  );
		lua_rpercent_check(mob->in_room, mob, RDEATH_PROG);
   }
   return;
}

int lua_ocustom_trigger( const char *command, const char *argument, CHAR_DATA * ch )
{
   ROOM_INDEX_DATA *room;
   MPROG_DATA *mprg;
   char pcom[MAX_STRING_LENGTH];
   char parg[MAX_STRING_LENGTH];
   char *temp;
   bool found = FALSE;
   OBJ_DATA *vobj;
   if( ( room = ch->in_room ) == NULL )
      return 0;
   for( vobj = room->first_content; vobj; vobj = vobj->next_content )
   {
      if( vobj != NULL && vobj->pIndexData != NULL )
      {
         for( mprg = vobj->pIndexData->mudprogs; mprg; mprg = mprg->next )
         {
            if( mprg->type & CUSTOM_PROG )
            {
               temp = str_dup( mprg->arglist );
               temp = one_argument( temp, pcom );
               temp = one_argument( temp, parg );
               if( !strcmp( pcom, command ) )
               {
                  if( parg[0] == '\0' )
                  {
                     found = TRUE;
                     break;
                  }
                  else if( !strcmp( parg, argument ) )
                  {
                     found = TRUE;
                     break;
                  }
               }
            }
         }
      }
      if( found )
         break;
   }
   if( !found )
   {
      for( vobj = ch->first_carrying; vobj; vobj = vobj->next_content )
      {
         if( vobj != NULL && vobj->pIndexData != NULL )
         {
            for( mprg = vobj->pIndexData->mudprogs; mprg; mprg = mprg->next )
            {
               if( mprg->type & CUSTOM_PROG )
               {
                  temp = str_dup( mprg->arglist );
                  temp = one_argument( temp, pcom );
                  temp = one_argument( temp, parg );
                  if( !strcmp( pcom, command ) )
                  {
                     if( parg[0] == '\0' )
                     {
                        found = TRUE;
                        break;
                     }
                     else if( !strcmp( parg, argument ) )
                     {
                        found = TRUE;
                        break;
                     }
                  }
               }
            }
         }
         if( found )
            break;
      }
   }
   if( !found )
      return 0;
//   set_supermob( vobj );
//   mprog_driver( mprg->comlist, supermob, ch, vobj, NULL, FALSE );
//   set_supermob( vobj );
	lua_hook(mprg->comlist, CUSTOM_PROG, ch, NULL, vobj, NULL);
   return 1;
}

int lua_mcustom_trigger( const char *command, const char *argument, CHAR_DATA * ch )
{
   ROOM_INDEX_DATA *room;
   MPROG_DATA *mprg;
   char pcom[MAX_STRING_LENGTH];
   char parg[MAX_STRING_LENGTH];
   char *temp;
   bool found = FALSE;
   CHAR_DATA *vmob;
   if( ( room = ch->in_room ) == NULL )
      return 0;
   for( vmob = room->first_person; vmob; vmob = vmob->next_in_room )
   {
      if( vmob != NULL && vmob->pIndexData != NULL )
         for( mprg = vmob->pIndexData->mudprogs; mprg; mprg = mprg->next )
         {
            if( mprg->type & CUSTOM_PROG )
            {
               temp = str_dup( mprg->arglist );
               temp = one_argument( temp, pcom );
               temp = one_argument( temp, parg );
               if( !strcmp( pcom, command ) )
               {
                  if( parg[0] == '\0' )
                  {
                     found = TRUE;
                     break;
                  }
                  else if( !strcmp( parg, argument ) )
                  {
                     found = TRUE;
                     break;
                  }
               }
            }
         }
      if( found )
         break;
   }
   if( !found )
      return 0;
//   mprog_driver( mprg->comlist, vmob, ch, NULL, NULL, FALSE );
	lua_hook(mprg->comlist, CUSTOM_PROG, ch, vmob, NULL, NULL);
   return 1;
}

int lua_rcustom_trigger( const char *command, const char *argument, CHAR_DATA * ch )
{
   ROOM_INDEX_DATA *room;
   MPROG_DATA *mprg;
   char pcom[MAX_STRING_LENGTH];
   char parg[MAX_STRING_LENGTH];
   char *temp;
   bool found = FALSE;
   if( ( room = ch->in_room ) == NULL )
      return 0;
   for( mprg = room->mudprogs; mprg; mprg = mprg->next )
   {
      if( mprg->type & CUSTOM_PROG )
      {
         temp = str_dup( mprg->arglist );
         temp = one_argument( temp, pcom );
         temp = one_argument( temp, parg );
         if( !strcmp( pcom, command ) )
         {
            if( parg[0] == '\0' )
            {
               found = TRUE;
               break;
            }
            else if( !strcmp( parg, argument ) )
            {
               found = TRUE;
               break;
            }
         }
      }
   }
   if( !found )
      return 0;
//   rset_supermob( room );
//   mprog_driver( mprg->comlist, supermob, ch, NULL, NULL, FALSE );
//   release_supermob(  );
	lua_hook(mprg->comlist, CUSTOM_PROG, ch, NULL, NULL, room);
   return 1;
}

void lua_mfight_trigger( CHAR_DATA * mob, CHAR_DATA * ch )
{

   if( IS_NPC( mob ) && ( mob->pIndexData->progtypes & FIGHT_PROG ) )
      lua_mpercent_check( mob, ch, FIGHT_PROG );

   return;
}

void lua_mhitprcnt_trigger(CHAR_DATA * ch, CHAR_DATA * mob)
{
	MPROG_DATA * mprg;

	if (IS_NPC(mob) && mob->pIndexData->progtypes & HITPRCNT_PROG)
		for (mprg = mob->pIndexData->mudprogs; mprg; mprg = mprg->next)
			if ((mprg->type & HITPRCNT_PROG) && ((100 * mob->hit / mob->max_hit) < atoi(mprg->arglist)))
			{
				lua_hook(mprg->comlist, HITPRCNT_PROG, ch, mob, NULL, NULL);
				break;
			}

	return;
}

void lua_rfight_trigger(CHAR_DATA * ch)
{
	ROOM_INDEX_DATA * room = ch->in_room;
	if (room->progtypes & RFIGHT_PROG)
		lua_rpercent_check(room, ch, RFIGHT_PROG);

	return;
}

void lua_mgive_trigger( CHAR_DATA * mob, CHAR_DATA * ch, OBJ_DATA * obj )
{
   char buf[MAX_INPUT_LENGTH];
   MPROG_DATA *mprg;

   if( IS_NPC( mob ) && ( mob->pIndexData->progtypes & GIVE_PROG ) )
   {
      /*
       * Don't let a mob trigger itself, nor one instance of a mob
       * trigger another instance.
       */
      if( IS_NPC( ch ) && ch->pIndexData == mob->pIndexData )
         return;

      for( mprg = mob->pIndexData->mudprogs; mprg; mprg = mprg->next )
      {
         one_argument( mprg->arglist, buf );

         if( ( mprg->type & GIVE_PROG ) && ( ( !str_cmp( obj->name, mprg->arglist ) ) || ( !str_cmp( "all", buf ) ) ) )
         {
//            mprog_driver( mprg->comlist, mob, ch, obj, NULL, FALSE );
				lua_hook(mprg->comlist, GIVE_PROG, ch, mob, obj, NULL);
            break;
         }
      }
   }
   return;
}

void lua_rsleep_trigger( CHAR_DATA * ch )
{
	ROOM_INDEX_DATA * room = ch->in_room;

   if (room->progtypes & SLEEP_PROG)
   {
//      rset_supermob( ch->in_room );
//      rprog_percent_check( supermob, ch, NULL, NULL, SLEEP_PROG );
//      release_supermob(  );
		lua_rpercent_check(room, ch, SLEEP_PROG);
   }
   return;
}

void lua_rrest_trigger( CHAR_DATA * ch )
{
   if( ch->in_room->progtypes & REST_PROG )
   {
//      rset_supermob( ch->in_room );
//      rprog_percent_check( supermob, ch, NULL, NULL, REST_PROG );
//      release_supermob(  );
		lua_rpercent_check(ch->in_room, ch, REST_PROG);
   }
   return;
}

void lua_mrandom_trigger(CHAR_DATA * mob)
{
	if (IS_NPC(mob) && mob->pIndexData->progtypes & RAND_PROG)
		lua_mpercent_check(mob, NULL, RAND_PROG);

	return;
}

void lua_rrandom_trigger(CHAR_DATA * ch)
{
	ROOM_INDEX_DATA * room = ch->in_room;
	if (room->progtypes & RAND_PROG)
		lua_rpercent_check(room, ch, RAND_PROG);

	return;
}

void lua_orandom_trigger(OBJ_DATA * obj)
{
	if (obj->pIndexData->progtypes & RAND_PROG)
		lua_opercent_check(obj, obj->carried_by, RAND_PROG);

	return;
}

void lua_mtime_trigger( CHAR_DATA * mob )
{
   if (IS_NPC(mob) && mob->pIndexData->progtypes & TIME_PROG)
      lua_mtime_check( mob, TIME_PROG );

   return;
}

void lua_mhour_trigger( CHAR_DATA * mob )
{
   if( mob->pIndexData->progtypes & HOUR_PROG )
      lua_mtime_check( mob, HOUR_PROG );

   return;
}

void lua_rtime_trigger( CHAR_DATA * mob )
{
	ROOM_INDEX_DATA * room = mob->in_room;
   if (room->progtypes & TIME_PROG)
      lua_mtime_check( mob, TIME_PROG );

   return;
}

void lua_rhour_trigger( CHAR_DATA * mob )
{
   if( mob->in_room->progtypes & HOUR_PROG )
      lua_mtime_check( mob, HOUR_PROG );

   return;
}

void lua_mspeech_trigger( const char *txt, CHAR_DATA * actor )
{

   CHAR_DATA *vmob;

   for( vmob = actor->in_room->first_person; vmob; vmob = vmob->next_in_room )
   {
      if( IS_NPC( vmob ) && ( vmob->pIndexData->progtypes & SPEECH_PROG ) )
      {
         if( IS_NPC( actor ) && actor->pIndexData == vmob->pIndexData )
            continue;
//         mprog_wordlist_check( txt, vmob, actor, NULL, NULL, SPEECH_PROG );
			lua_mwordlist_check(txt, vmob, actor, NULL, NULL, SPEECH_PROG);
      }
   }

   return;
}

void lua_ospeech_trigger( const char *txt, CHAR_DATA * ch )
{
   OBJ_DATA *vobj;

   /*
    * supermob is set and released in oprog_wordlist_check
    */
   for( vobj = ch->in_room->first_content; vobj; vobj = vobj->next_content )
      if( vobj->pIndexData->progtypes & SPEECH_PROG )
      {
//         oprog_wordlist_check( txt, supermob, ch, vobj, NULL, SPEECH_PROG, vobj );
			lua_owordlist_check(txt, NULL, ch, vobj, NULL, SPEECH_PROG, vobj);
      }

   return;
}

void lua_rspeech_trigger( const char *txt, CHAR_DATA * ch )
{
   if( ch->in_room->progtypes & SPEECH_PROG )
   {
      /*
       * supermob is set and released in rprog_wordlist_check
       */
//      rprog_wordlist_check( txt, supermob, ch, NULL, NULL, SPEECH_PROG, ch->in_room );
		lua_rwordlist_check(txt, NULL, ch, NULL, NULL, SPEECH_PROG, ch->in_room);
   }
   return;
}

void lua_owear_trigger( CHAR_DATA * ch, OBJ_DATA * obj )
{
   if( obj->pIndexData->progtypes & WEAR_PROG )
   {
//      set_supermob( obj );
//      oprog_percent_check( supermob, ch, obj, NULL, WEAR_PROG );
//      release_supermob(  );
		lua_opercent_check(obj, ch, WEAR_PROG);
   }
   return;
}

bool lua_ouse_trigger( CHAR_DATA * ch, OBJ_DATA * obj, CHAR_DATA * vict )
{
   bool executed = FALSE;

   if( obj->pIndexData->progtypes & USE_PROG )
   {
//      set_supermob( obj );
		executed = lua_ouse_check(obj, ch, vict, USE_PROG);
/*      if( obj->item_type == ITEM_STAFF )
      {
         if( vict )
            executed = oprog_percent_check( supermob, ch, obj, vict, USE_PROG );
         else
            executed = oprog_percent_check( supermob, ch, obj, targ, USE_PROG );
      }
      else
      {
         executed = oprog_percent_check( supermob, ch, obj, NULL, USE_PROG );
      } */
//      release_supermob(  );
   }
   return executed;
}

void lua_oremove_trigger( CHAR_DATA * ch, OBJ_DATA * obj )
{
   if( obj->pIndexData->progtypes & REMOVE_PROG )
   {
//      set_supermob( obj );
//      oprog_percent_check( supermob, ch, obj, NULL, REMOVE_PROG );
//      release_supermob(  );
		lua_opercent_check(obj, ch, REMOVE_PROG);
   }
   return;
}

void lua_osac_trigger( CHAR_DATA * ch, OBJ_DATA * obj )
{
   if( obj->pIndexData->progtypes & SAC_PROG )
   {
//      set_supermob( obj );
//      oprog_percent_check( supermob, ch, obj, NULL, SAC_PROG );
//      release_supermob(  );
		lua_opercent_check(obj, ch, SAC_PROG);
   }
   return;
}

void lua_oget_trigger( CHAR_DATA * ch, OBJ_DATA * obj )
{
   if( obj->pIndexData->progtypes & GET_PROG )
   {
//      set_supermob( obj );
//      oprog_percent_check( supermob, ch, obj, NULL, GET_PROG );
//      release_supermob(  );
		lua_opercent_check(obj, ch, GET_PROG);
   }
   return;
}

void lua_odamage_trigger( CHAR_DATA * ch, OBJ_DATA * obj )
{
   if( obj->pIndexData->progtypes & DAMAGE_PROG )
   {
//      set_supermob( obj );
//      oprog_percent_check( supermob, ch, obj, NULL, DAMAGE_PROG );
//      release_supermob(  );
		lua_opercent_check(obj, ch, DAMAGE_PROG);
   }
   return;
}

void lua_orepair_trigger( CHAR_DATA * ch, OBJ_DATA * obj )
{

   if( obj->pIndexData->progtypes & REPAIR_PROG )
   {
//      set_supermob( obj );
//      oprog_percent_check( supermob, ch, obj, NULL, REPAIR_PROG );
//      release_supermob(  );
		lua_opercent_check(obj, ch, REPAIR_PROG);
   }
   return;
}

void lua_odrop_trigger( CHAR_DATA * ch, OBJ_DATA * obj )
{
   if( obj->pIndexData->progtypes & DROP_PROG )
   {
//      set_supermob( obj );
//      oprog_percent_check( supermob, ch, obj, NULL, DROP_PROG );
//      release_supermob(  );
		lua_opercent_check(obj, ch, DROP_PROG);
   }
   return;
}

void lua_oexamine_trigger( CHAR_DATA * ch, OBJ_DATA * obj )
{
   if( obj->pIndexData->progtypes & EXA_PROG )
   {
//      set_supermob( obj );
//      oprog_percent_check( supermob, ch, obj, NULL, EXA_PROG );
//      release_supermob(  );
		lua_opercent_check(obj, ch, EXA_PROG);
   }
   return;
}

void lua_mlook_trigger(CHAR_DATA * ch, CHAR_DATA * mob)
{
	if (IS_NPC(mob) && mob->pIndexData->progtypes & LOOK_PROG)
		lua_mpercent_check(mob, ch, LOOK_PROG);

	return;
}

void lua_rlook_trigger(CHAR_DATA * ch)
{
	if (ch->in_room->progtypes & LOOK_PROG)
		lua_rpercent_check(ch->in_room, ch, LOOK_PROG);

	return;
}

