// Lua interface.
// Copyright (c) 2012 by Kasji.

#include <stdio.h>
#include <string.h>
#include "lua.h"

extern tPointerOwner<CHAR_DATA> char_map;
extern tPointerOwner<OBJ_DATA> obj_map;
extern tPointerOwner<ROOM_INDEX_DATA> room_map;

extern const char * const dam_type_table[];

const char * const luaCHMetas[] =
{
	"name", "short_desc", "long_desc", "description", "gender", "race", "inventory",
	"top_level", "ability", "bonus_ability", "trust", "played", "logon", "save_time",
	"hit", "max_hit", "move", "max_move", "mana", "max_mana", "credits", "affected_by",
	"carry_weight", "carry_number", "speaks", "speaking", "align", "hitroll",
	"damroll", "position", "wimpy", "deaf", "str", "int", "wis", "dex", "con", "cha",
	"lck", "frc", "mod_str", "mod_int", "mod_wis", "mod_dex", "mod_con", "mod_cha",
	"mod_lck", "mod_frc", "vip_flags", "main_ability", "secondary_ability", "comfreq",
	"rank", "base_res", "curr_res", "encumbrance", "sh_deflect", "dmg_absorb",
	"size_modifier", "vnum"
};
// The above strings NEED to match the order of the below enumeration.
enum luaCHMetasEnum
{
	CH_NAME=0, CH_SHORT_DESC, CH_LONG_DESC, CH_DESCRIPTION, CH_GENDER, CH_RACE, CH_INVENTORY,
	CH_TOP_LEVEL, CH_ABILITY, CH_BONUS_ABILITY, CH_TRUST, CH_PLAYED, CH_LOGON, CH_SAVE_TIME,
	CH_HIT, CH_MAX_HIT, CH_MOVE, CH_MAX_MOVE, CH_MANA, CH_MAX_MANA, CH_CREDITS, CH_AFFECTED_BY,
	CH_CARRY_WEIGHT, CH_CARRY_NUMBER, CH_SPEAKS, CH_SPEAKING, CH_ALIGNMENT, CH_HITROLL,
	CH_DAMROLL, CH_POSITION, CH_WIMPY, CH_DEAF, CH_STR, CH_INT, CH_WIS, CH_DEX, CH_CON, CH_CHA,
	CH_LCK, CH_FRC, CH_MOD_STR, CH_MOD_INT, CH_MOD_WIS, CH_MOD_DEX, CH_MOD_CON, CH_MOD_CHA,
	CH_MOD_LCK, CH_MOD_FRC, CH_VIP_FLAGS, CH_MAIN_ABILITY, CH_SECONDARY_ABILITY, CH_COMFREQ,
	CH_RANK, CH_BASE_RES, CH_CURR_RES, CH_ENCUMBRANCE, CH_SH_DEFLECT, CH_DMG_ABSORB,
	CH_SIZE_MODIFIER, CH_VNUM,

	MAX_CHMETAS
};

const char * const luaRoomMetas[] =
{
	"name", "description", "vnum", "flags", "light", "sector", "tele_vnum", "tele_delay", "tunnel"
};
// The above strings NEED to match the order of the below enumeration.
enum luaRoomMetasEnum
{
	ROOM_NAME, ROOM_DESCRIPTION, ROOM_VNUM, ROOM_FLAGS, ROOM_LIGHT, ROOM_SECTOR, ROOM_TELE_VNUM, ROOM_TUNNEL, ROOM_TELE_DELAY,

	MAX_ROOMMETAS
};

const char * const luaObjMetas[] =
{
	"name", "short_desc", "description", "action_desc", "item_type", "extra_flags", "magic_flags"
	"wear_flags", "blaster_setting", "wear_loc", "weight", "cost", "level", "value", "count",
	"dam_type", "vnum"
};
// The above strings NEED to match the order of the below enumeration.
enum luaObjMetasEnum
{
	OBJ_NAME, OBJ_SHORT_DESC, OBJ_DESCRIPTION, OBJ_ACTION_DESC, OBJ_ITEM_TYPE, OBJ_EXTRA_FLAGS, OBJ_MAGIC_FLAGS,
	OBJ_WEAR_FLAGS, OBJ_BLASTER_SETTING, OBJ_WEAR_LOC, OBJ_WEIGHT, OBJ_COST, OBJ_LEVEL, OBJ_VALUE, OBJ_COUNT,
	OBJ_DAM_TYPE, OBJ_VNUM,

	MAX_OBJMETAS
};

extern "C" int luaMudGet(lua_State * L)
{
	// For now until finished...
	lua_pushnil(L);
	return 1;
}

// Ch read metamethod.
extern "C" int luaCHGet(lua_State * L)
{
	const char * field = lua_tostring(L, -1); // Name of the field being called.
	int i;
	uint64 id = 0;
	CHAR_DATA * ch;

	lua_getfield(L, -2, "id"); // ID of the character
	id = (uint64)lua_touserdata(L, -1);
	ch = char_map[id];
	// No character was found?
	if (!ch)
	{
		lua_pushnil(L);
		return 1;
	}

	// Check to see if the field being called actually exists.
	for (i = 0; i < MAX_CHMETAS; i++)
		if (!str_cmp(field, luaCHMetas[i]))
			break;

	switch (i)
	{
	case CH_NAME:
		// Add introduction system support
		if (ch->name)
			lua_pushstring(L, ch->name);
		else
			lua_pushnil(L);
		return 1;
	case CH_SHORT_DESC:
		if (ch->short_descr)
			lua_pushstring(L, ch->short_descr);
		else
			lua_pushnil(L);
		return 1;
	case CH_LONG_DESC:
		if (ch->long_descr)
			lua_pushstring(L, ch->long_descr);
		else
			lua_pushnil(L);
		return 1;
	case CH_DESCRIPTION:
		if (ch->description)
			lua_pushstring(L, ch->description);
		else
			lua_pushnil(L);
		return 1;
	case CH_GENDER:
		lua_pushstring(L, ch->sex == 1 ? "male" : ch->sex == 2 ? "female" : "neutral");
		return 1;
	case CH_RACE:
		lua_pushstring(L, get_race(ch));
		return 1;
	case CH_INVENTORY:
		if (ch->first_carrying)
		{
			lua_newtable(L);
		}
		else
			lua_pushnil(L);
		return 1;
	case CH_TOP_LEVEL:
		lua_pushnumber(L, ch->top_level);
		return 1;
	case CH_ABILITY:
/*		array = lua_tonumber(L, -1);
		if (array < 0 || array >= MAX_ABILITY)
			lua_pushnil(L);
		else
			lua_pushnumber(L, ch->skill_level[array]); */
		// This needs it's own metamethod and metatable.
		lua_pushnil(L);
		return 1;
	case CH_BONUS_ABILITY:
/*		array = lua_tonumber(L, -1);
		if (array < 0 || array >= MAX_ABILITY)
			lua_pushnil(L);
		else
			lua_pushnumber(L, ch->bonus[array]); */
		// This needs it's own metamethod and metatable.
		lua_pushnil(L);
		return 1;
	case CH_TRUST:
		lua_pushnumber(L, ch->trust);
		return 1;
	case CH_PLAYED:
		lua_pushnumber(L, ch->played);
		return 1;
	case CH_LOGON:
		lua_pushnumber(L, ch->logon);
		return 1;
	case CH_SAVE_TIME:
		lua_pushnumber(L, ch->save_time);
		return 1;
	case CH_HIT:
		lua_pushnumber(L, ch->hit);
		return 1;
	case CH_MAX_HIT:
		lua_pushnumber(L, ch->max_hit);
		return 1;
	case CH_MOVE:
		lua_pushnumber(L, ch->move);
		return 1;
	case CH_MAX_MOVE:
		lua_pushnumber(L, ch->max_move);
		return 1;
	case CH_MANA:
		lua_pushnumber(L, ch->mana);
		return 1;
	case CH_MAX_MANA:
		lua_pushnumber(L, ch->max_mana);
		return 1;
	case CH_CREDITS:
		lua_pushnumber(L, ch->gold);
		return 1;
	case CH_AFFECTED_BY:
		lua_pushnumber(L, ch->affected_by);
		return 1;
	case CH_CARRY_WEIGHT:
		lua_pushnumber(L, ch->carry_weight);
		return 1;
	case CH_CARRY_NUMBER:
		lua_pushnumber(L, ch->carry_number);
		return 1;
	case CH_SPEAKS:
		lua_pushnumber(L, ch->speaks);
		return 1;
	case CH_SPEAKING:
		lua_pushnumber(L, ch->speaking);
		return 1;
	case CH_ALIGNMENT:
		lua_pushnumber(L, ch->alignment);
		return 1;
	case CH_HITROLL:
		lua_pushnumber(L, GET_HITROLL(ch));
		return 1;
	case CH_DAMROLL:
		lua_pushnumber(L, GET_DAMROLL(ch));
		return 1;
	case CH_POSITION:
		lua_pushnumber(L, ch->position);
		return 1;
	case CH_WIMPY:
		lua_pushnumber(L, ch->wimpy);
		return 1;
	case CH_DEAF:
		lua_pushnumber(L, ch->deaf);
		return 1;
   case CH_STR:
      lua_pushnumber(L, ch->perm_str);
      return 1;
   case CH_INT:
      lua_pushnumber(L, ch->perm_int);
      return 1;
   case CH_WIS:
      lua_pushnumber(L, ch->perm_wis);
      return 1;
   case CH_DEX:
      lua_pushnumber(L, ch->perm_dex);
      return 1;
   case CH_CON:
      lua_pushnumber(L, ch->perm_con);
      return 1;
   case CH_CHA:
      lua_pushnumber(L, ch->perm_cha);
      return 1;
   case CH_LCK:
      lua_pushnumber(L, ch->perm_lck);
      return 1;
   case CH_FRC:
      lua_pushnumber(L, ch->perm_frc);
      return 1;
   case CH_MOD_STR:
      lua_pushnumber(L, ch->mod_str);
      return 1;
   case CH_MOD_INT:
      lua_pushnumber(L, ch->mod_int);
      return 1;
   case CH_MOD_WIS:
      lua_pushnumber(L, ch->mod_wis);
      return 1;
   case CH_MOD_DEX:
      lua_pushnumber(L, ch->mod_dex);
      return 1;
   case CH_MOD_CON:
      lua_pushnumber(L, ch->mod_con);
      return 1;
   case CH_MOD_CHA:
      lua_pushnumber(L, ch->mod_cha);
      return 1;
   case CH_MOD_LCK:
      lua_pushnumber(L, ch->mod_lck);
      return 1;
   case CH_MOD_FRC:
      lua_pushnumber(L, ch->mod_frc);
      return 1;
   case CH_VIP_FLAGS:
      lua_pushnumber(L, ch->vip_flags);
      return 1;
   case CH_MAIN_ABILITY:
      lua_pushnumber(L, ch->main_ability);
      return 1;
   case CH_SECONDARY_ABILITY:
      lua_pushnumber(L, ch->secondary_ability);
      return 1;
   case CH_COMFREQ:
      if (ch->comfreq)
         lua_pushstring(L, ch->comfreq);
      else
         lua_pushnil(L);
      return 1;
   case CH_RANK:
      if (ch->rank)
         lua_pushstring(L, ch->rank);
      else
         lua_pushnil(L);
      return 1;
   case CH_BASE_RES:
/*		field2 = lua_tostring(L, -1);
		for (array = 0; array < RES_MAX; array++)
	         	if (!str_cmp(field2, dam_type_table[array]))
				break;
		if (array >= RES_MAX)
			lua_pushnil(L);
		else
			lua_pushnumber(L, ch->base_res[array]); */
		// This needs it's own metamethod and metatable.
		lua_pushnil(L);
		return 1;
	case CH_CURR_RES:
/*		field2 = lua_tostring(L, -1);
for (array = 0; array < RES_MAX; array++)
        	        if (!str_cmp(field2, dam_type_table[array]))
                	        break;
                if (array >= RES_MAX)
                        lua_pushnil(L);
                else
                        lua_pushnumber(L, ch->curr_res[array]); */
		// This needs it's own metamethod and metatable.
		lua_pushnil(L);
		return 1;
   case CH_ENCUMBRANCE:
      lua_pushnumber(L, ch->encumbrance);
      return 1;
   case CH_SH_DEFLECT:
      lua_pushnumber(L, ch->sh_deflect);
      return 1;
   case CH_DMG_ABSORB:
      lua_pushnumber(L, ch->dmg_absorb);
      return 1;
   case CH_SIZE_MODIFIER:
      lua_pushnumber(L, ch->size_modifier);
      return 1;
	case CH_VNUM:
		if (ch->pIndexData)
			lua_pushnumber(L, ch->pIndexData->vnum);
		else
			lua_pushnil(L);
		return 1;
   case MAX_CHMETAS:
      lua_pushnil(L);
      return 1;
	default:
		// Shouldn't ever make it here, but just in case...
		lua_pushnil(L);
		return 1;
	}

	return 0;
}

// Ch write metamethod.
extern "C" int luaCHSet(lua_State * L)
{
	return 0;
}

// Room read metamethod.
extern "C" int luaRoomGet(lua_State * L)
{
	const char * field = lua_tostring(L, -1); // Name of the field being called.
	int i;
	uint64 id = 0;
	ROOM_INDEX_DATA * room;

	lua_getfield(L, -2, "id");
	id = (uint64)lua_touserdata(L, -1);
	room = room_map[id];
	// No room was found?
	if (!room)
	{
		lua_pushnil(L);
		return 1;
	}

	// Check to see if the field being called actually exists.
	for (i = 0; i < MAX_ROOMMETAS; i++)
		if (!str_cmp(field, luaRoomMetas[i]))
			break;

	switch (i)
	{
   case ROOM_NAME:
		if (room->name)
			lua_pushstring(L, room->name);
		else
			lua_pushnil(L);
		return 1;
	case ROOM_DESCRIPTION:
		if (room->description)
			lua_pushstring(L, room->description);
		else
			lua_pushnil(L);
		return 1;
	case ROOM_VNUM:
		lua_pushnumber(L, room->vnum);
		return 1;
	case ROOM_FLAGS:
		lua_newtable(L);
		return 1;
	case ROOM_LIGHT:
		lua_pushnumber(L, room->light);
	case ROOM_SECTOR:
		lua_pushnumber(L, room->sector_type);
		return 1;
	case ROOM_TELE_VNUM:
		lua_pushnumber(L, room->tele_vnum);
		return 1;
	case ROOM_TELE_DELAY:
		lua_pushnumber(L, room->tele_delay);
		return 1;
	case ROOM_TUNNEL:
		lua_pushnumber(L, room->tunnel);
		return 1;
	case MAX_ROOMMETAS:
		lua_pushnil(L);
		return 1;
	default:
		lua_pushnil(L);
		return 1;
	}

	return 0;
}

// Room write metamethod.
extern "C" int luaRoomSet(lua_State * L)
{
	return 0;
}

// Obj read metamethod.
extern "C" int luaObjGet(lua_State * L)
{
	const char * field = lua_tostring(L, -1); // Name of the field being called.
	int i;
	uint64 id = 0;
	OBJ_DATA * obj;

	lua_getfield(L, -2, "id");
	id = (uint64)lua_touserdata(L, -1);
	obj = obj_map[id];
	// No object was found?
	if (!obj)
	{
		lua_pushnil(L);
		return 1;
	}

	// Check to see if the field being called actually exists.
	for (i = 0; i < MAX_OBJMETAS; i++)
		if (!str_cmp(field, luaObjMetas[i]))
			break;

   switch (i)
   {
	case OBJ_NAME:
		if (obj->name)
			lua_pushstring(L, obj->name);
		else
			lua_pushnil(L);
		return 1;
	case OBJ_SHORT_DESC:
		if (obj->short_descr)
			lua_pushstring(L, obj->short_descr);
		else
			lua_pushnil(L);
		return 1;
	case OBJ_DESCRIPTION:
		if (obj->description)
         lua_pushstring(L, obj->description);
		else
         lua_pushnil(L);
		return 1;
	case OBJ_ACTION_DESC:
		if (obj->action_desc)
         lua_pushstring(L, obj->action_desc);
		else
         lua_pushnil(L);
		return 1;
	case OBJ_ITEM_TYPE:
		lua_pushnumber(L, obj->item_type);
		return 1;
	case OBJ_EXTRA_FLAGS:
		lua_pushnumber(L, obj->extra_flags);
		return 1;
	case OBJ_MAGIC_FLAGS:
		lua_pushnumber(L, obj->magic_flags);
		return 1;
	case OBJ_WEAR_FLAGS:
		lua_pushnumber(L, obj->wear_flags);
		return 1;
	case OBJ_BLASTER_SETTING:
		lua_pushnumber(L, obj->blaster_setting);
		return 1;
	case OBJ_WEAR_LOC:
		lua_pushnumber(L, obj->wear_loc);
		return 1;
	case OBJ_WEIGHT:
		lua_pushnumber(L, obj->weight);
		return 1;
	case OBJ_COST:
		lua_pushnumber(L, obj->cost);
		return 1;
	case OBJ_LEVEL:
		lua_pushnumber(L, obj->level);
		return 1;
	case OBJ_VALUE:
		/* array = lua_tonumber(L, -1);
		if (array < 0 || array >= 6)
			lua_pushnil(L);
		else
         lua_pushnumber(L, obj->value[array]); */
		// Requires its own metatable and method.
		lua_pushnil(L);
      return 1;
	case OBJ_COUNT:
		lua_pushnumber(L, obj->count);
		return 1;
	case OBJ_DAM_TYPE:
		if (obj->item_type == ITEM_WEAPON && obj->dam_type >= 0 && obj->dam_type < RES_MAX)
	   	lua_pushstring(L, dam_type_table[obj->dam_type]);
		else
			lua_pushnil(L);
		return 1;
	case OBJ_VNUM:
		if (obj->pIndexData)
			lua_pushnumber(L, obj->pIndexData->vnum);
		else
			lua_pushnil(L);
		return 1;
	case MAX_OBJMETAS:
		lua_pushnil(L);
		return 1;
	default:
		// Shouldn't ever make it here, but just in case...
		lua_pushnil(L);
		return 1;
	}
	return 0;
}

// Obj write metamethod.
extern "C" int luaObjSet(lua_State * L)
{
        return 0;
}

