#include <stdio.h>
#include <string.h>
#include "mud.h"
#include "lua.h"

extern tPointerOwner<CHAR_DATA> char_map;
extern tPointerOwner<OBJ_DATA> obj_map;
extern tPointerOwner<ROOM_INDEX_DATA> room_map;
extern tPointerOwner<L_COROUTINE> coroutine_map;

template <class ITER, class F>
F safe_for_each_ch(CHAR_DATA * ch, ITER first, ITER last, F func)
  {
  while (first != last)
    func (ch, *first++);
  return func;
  }

void list_char_map(CHAR_DATA * ch, const std::pair <tId<CHAR_DATA>,CHAR_DATA*> item)
{
   if (item.second->pIndexData)
      pager_printf(ch, "&g| &W%-10d &g|| &W%-30s &g|| &W%-10d &g|\n\r", item.first.Value(), item.second->name, item.second->pIndexData->vnum);
   else
      pager_printf(ch, "&g| &W%-10d &g|| &W%-30s &g|| &W%-10s &g|\n\r", item.first.Value(), item.second->name, " ");
   return;
}

void list_obj_map(CHAR_DATA * ch, const std::pair <tId<OBJ_DATA>,OBJ_DATA*> item)
{
   pager_printf(ch, "&g| &W%-10d &g|| &W%-30s &g|| &W%-10d &g|\n\r", item.first.Value(), item.second->name, item.second->pIndexData->vnum);
   return;
}

void list_room_map(CHAR_DATA * ch, const std::pair <tId<ROOM_INDEX_DATA>,ROOM_INDEX_DATA*> item)
{
   pager_printf(ch, "&g| &W%-10d &g|| &W%-30s &g|| &W%-10d &g|\n\r", item.first.Value(), item.second->name, item.second->vnum);
   return;
}

void list_coroutine_map(CHAR_DATA * ch, const std::pair <tId<L_COROUTINE>,L_COROUTINE*> item)
{
	const char * stat;
	if (item.second->status() == LUA_YIELD)
		stat = "yielded";
	else if (item.second->status() == 0)
		stat = "finished";
	else
		stat = "errored";
	pager_printf(ch, "&g| &W%-10d &g|| &W%10s&g-&W%-10d &g|| &W%-32s &g|| &W%-8s &g|\n\r", item.first.Value(), item.second->owner_type(),
			item.second->owner_id(), item.second->func(), stat);
	return;
}


void do_charmap(CHAR_DATA * ch, const char * argument)
{
   bool force_pager = FALSE;
   if (IS_NPC(ch))
      return;
   if (!IS_SET(ch->pcdata->flags, PCFLAG_PAGERON))
   {
      force_pager = TRUE;
      SET_BIT(ch->pcdata->flags, PCFLAG_PAGERON);
   }
   pager_printf(ch, "&g##############################################################\n\r");
   pager_printf(ch, "&g|     &WID     &g||              &WName              &g||    &WVnum    &g|\n\r");
   pager_printf(ch, "&g##############################################################\n\r");
   safe_for_each_ch(ch, char_map.begin(), char_map.end(), list_char_map);
   pager_printf(ch, "&g##############################################################\n\r");
   if (force_pager == TRUE)
      REMOVE_BIT(ch->pcdata->flags, PCFLAG_PAGERON);
   return;
}

void do_objmap(CHAR_DATA * ch, const char * argument)
{
   bool force_pager = FALSE;
   if (IS_NPC(ch))
      return;
   if (!IS_SET(ch->pcdata->flags, PCFLAG_PAGERON))
   {
      force_pager = TRUE;
      SET_BIT(ch->pcdata->flags, PCFLAG_PAGERON);
   }
   pager_printf(ch, "&g##############################################################\n\r");
   pager_printf(ch, "&g|     &WID     &g||              &WName              &g||    &WVnum    &g|\n\r");
   pager_printf(ch, "&g##############################################################\n\r");
   safe_for_each_ch(ch, obj_map.begin(), obj_map.end(), list_obj_map);
   pager_printf(ch, "&g##############################################################\n\r");
   if (force_pager == TRUE)
      REMOVE_BIT(ch->pcdata->flags, PCFLAG_PAGERON);
   return;
}

void do_roommap(CHAR_DATA * ch, const char * argument)
{
   bool force_pager = FALSE;
   if (IS_NPC(ch))
      return;
   if (!IS_SET(ch->pcdata->flags, PCFLAG_PAGERON))
   {
      force_pager = TRUE;
      SET_BIT(ch->pcdata->flags, PCFLAG_PAGERON);
   }
   pager_printf(ch, "&g##############################################################\n\r");
   pager_printf(ch, "&g|     &WID     &g||              &WName              &g||    &WVnum    &g|\n\r");
   pager_printf(ch, "&g##############################################################\n\r");
   safe_for_each_ch(ch, room_map.begin(), room_map.end(), list_room_map);
   pager_printf(ch, "&g##############################################################\n\r");
   if (force_pager == TRUE)
      REMOVE_BIT(ch->pcdata->flags, PCFLAG_PAGERON);
   return;
}

void do_coroutinemap(CHAR_DATA * ch, const char * argument)
{
   bool force_pager = FALSE;
   if (IS_NPC(ch))
      return;
   if (!IS_SET(ch->pcdata->flags, PCFLAG_PAGERON))
   {
      force_pager = TRUE;
      SET_BIT(ch->pcdata->flags, PCFLAG_PAGERON);
   }
   pager_printf(ch, "&g#######################################################################################\n\r");
   pager_printf(ch, "&g|     &WID     &g|| &WOwner Type&g-&WID         &g||               &WName               &g||  &WStatus  &g|\n\r");
   pager_printf(ch, "&g#######################################################################################\n\r");
   safe_for_each_ch(ch, coroutine_map.begin(), coroutine_map.end(), list_coroutine_map);
   pager_printf(ch, "&g#######################################################################################\n\r");
   if (force_pager == TRUE)
      REMOVE_BIT(ch->pcdata->flags, PCFLAG_PAGERON);
   return;
}


